@@ -3,8 +3,7 @@
 
 from abc import ABC, abstractmethod
 from napoleontoolbox.file_saver import dropbox_file_saver
-import pandas as pd
-
+import talib as ta
 import numpy as np
 
 import torch
@@ -14,9 +13,30 @@ from os import walk
 import pandas as pd
 from pathlib import Path
 
-class AbstractAssembler(ABC):
+from tsfresh import extract_features
+from tsfresh.utilities.dataframe_functions import make_forecasting_frame
+from tsfresh.utilities.dataframe_functions import impute
 
-    def __init__(self, drop_token=None, returns_pkl_file_name='returns.pkl', features_pkl_file_name='features.pkl', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy', local_root_directory='../data/', user='napoleon',macro_features_directory = 'features_macro'):
+
+def buildAdvancedPriceFeatures(quotes_df=None, max_timeshift=20, rolling_direction=1):
+    strats = [col for col in quotes_df.columns if col != 'Date' ]
+    aggregated_features = None
+    for strat in strats:
+        print('processing '+strat)
+        df_shift, y = make_forecasting_frame(quotes_df[strat], kind="price", max_timeshift=20, rolling_direction=1)
+        X = extract_features(df_shift, column_id="id", column_sort="time", column_value="value", impute_function=impute, show_warnings=False)
+        X = X.loc[:, X.apply(pd.Series.nunique) != 1]
+        X.columns = [strat + col for col in X.columns]
+        if aggregated_features is None :
+            aggregated_features = X
+        else :
+            aggregated_features = pd.merge(aggregated_features, X, how = 'left', left_index=True, right_index=True)
+    return aggregated_features
+
+
+
+class AbstractAssembler(ABC):
+    def __init__(self, starting_date = None, running_date=None, drop_token=None, dropbox_backup=True, returns_pkl_file_name='returns.pkl', features_pkl_file_name='features.pkl', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy', local_root_directory='../data/', user='napoleon',macro_features_directory = 'features_macro',max_timeshift=20, rolling_direction=1):
         super().__init__()
         self.local_root_directory = local_root_directory
         self.user = user
@@ -25,7 +45,11 @@ class AbstractAssembler(ABC):
         self.features_names_saving_suffix = features_names_saving_suffix
         self.returns_pkl_file_name = returns_pkl_file_name
         self.macro_features_directory = macro_features_directory
-        self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token)
+        self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=drop_token,dropbox_backup=dropbox_backup)
+        self.running_date=running_date
+        self.starting_date=starting_date
+        self.max_timeshift = max_timeshift
+        self.rolling_direction = rolling_direction
 
     @abstractmethod
     def assembleFeature(self, feature_type, n_past_features):
@@ -33,7 +57,6 @@ class AbstractAssembler(ABC):
 
 
 class FeaturesAssembler(AbstractAssembler):
-
     def reassembleAdvancedFeaturesForClusterization(self, feature_type, n_past_features = 21, simple = True):
         if (feature_type is features_type.FeaturesType.HISTORY or feature_type is features_type.FeaturesType.HISTORY_ADVANCED) and n_past_features is None:
             return
@@ -47,14 +70,10 @@ class FeaturesAssembler(AbstractAssembler):
         torch.manual_seed(0)
 
         advanced_features, advanced_features_names = self.preprocessFeatures()
-        macro_features, macro_features_names = self.preprocessMacroFeatures()
         df, strats, strat_features_names = self.preprocessReturns()
 
-        duplicated_features = [my_feat for my_feat in advanced_features_names if my_feat in macro_features]
-        advanced_features = advanced_features.drop(columns=duplicated_features)
-        advanced_features_names = [col for col in list(advanced_features.columns) if col != 'Date']
 
-        features_names = advanced_features_names + macro_features_names
+        features_names = [col for col in list(advanced_features.columns) if col != 'Date']
 
 
         print(df.columns)
@@ -172,7 +191,7 @@ class FeaturesAssembler(AbstractAssembler):
 
     def preprocessFeatures(self):
 
-        features = self.dbx.local_overwrite_and_load_pickle( folder='', subfolder='', returns_pkl_file_name=self.features_pkl_file_name, local_root_directory = self.local_root_directory)
+        features = self.dbx.local_overwrite_and_load_pickle( folder='', subfolder=self.macro_features_directory, returns_pkl_file_name=self.features_pkl_file_name, local_root_directory = Path(self.local_root_directory))
         features_names = [col for col in list(features.columns) if col!='Date']
         features['Date'] = pd.to_datetime(features['Date'])
 
@@ -252,6 +271,51 @@ class FeaturesAssembler(AbstractAssembler):
         df = df.set_index('Date')
         df = df.fillna(method='ffill').fillna(method='bfill')
 
+        advanced_ts_features = buildAdvancedPriceFeatures(quotes_df=df[strats],max_timeshift=self.max_timeshift, rolling_direction=self.rolling_direction)
+
+        df = pd.merge(df, advanced_ts_features, how='left', left_index=True, right_index=True)
+
+        for col in strats:
+            df["ROC2_{}".format(col)] = df[col] / df[col].shift(2) - 1
+        for col in strats:
+            df["ROC3_{}".format(col)] = df[col] / df[col].shift(3) - 1
+        for col in strats:
+            df["ROC5_{}".format(col)] = df[col] / df[col].shift(5) - 1
+        for col in strats:
+            df["ROC10_{}".format(col)] = df[col] / df[col].shift(10) - 1
+        for col in strats:
+            df["ROC20_{}".format(col)] = df[col] / df[col].shift(20) - 1
+        for col in strats:
+            df["MM10_{}".format(col)] = df[col] / df[col].shift(1).rolling(window=10).mean() - 1
+        for col in strats:
+            df["MM20_{}".format(col)] = df[col] / df[col].shift(1).rolling(window=20).mean() - 1
+        for col in strats:
+            df["MM50_{}".format(col)] = df[col] / df[col].shift(1).rolling(window=50).mean() - 1
+        for col in strats:
+            df["MM100_{}".format(col)] = df[col] / df[col].shift(1).rolling(window=100).mean() - 1
+        for col in strats:
+            df["MM200_{}".format(col)] = df[col] / df[col].shift(1).rolling(window=200).mean() - 1
+        for col in strats:
+            df["RSI14_{}".format(col)] = ta.RSI(df[col], 14) / 50 - 1
+        for col in strats:
+            df["RSI27_{}".format(col)] = ta.RSI(df[col], 27) / 50 - 1
+        for col in strats:
+            df["RSI44_{}".format(col)] = ta.RSI(df[col], 44) / 50 - 1
+        for col in strats:
+            df["MACD_{}".format(col)] = ta.MACD(df[col])[1]
+
+        for col in strats:
+            df["HL21_{}".format(col)] = (df[col] - df[col].shift(1).rolling(window=21).min()) / \
+                                             (df[col].shift(1).rolling(window=21).max() - \
+                                              df[col].shift(1).rolling(window=21).min()) * 2 - 1
+        for col in strats:
+            df["HL50_{}".format(col)] = (df[col] - df[col].shift(1).rolling(window=50).min()) / \
+                                             (df[col].shift(1).rolling(window=50).max() - \
+                                              df[col].shift(1).rolling(window=50).min()) * 2 - 1
+        for col in strats:
+            df["HL200_{}".format(col)] = (df[col] - df[col].shift(1).rolling(window=200).min()) / \
+                                              (df[col].shift(1).rolling(window=200).max() - \
+                                               df[col].shift(1).rolling(window=200).min()) * 2 - 1
         # MM 3 mois
         for col in strats:
             df["MM63_{}".format(col)] = df[col] / df[col].shift(1).rolling(window=63).mean() - 1
@@ -332,14 +396,18 @@ class FeaturesAssembler(AbstractAssembler):
         torch.manual_seed(0)
 
         advanced_features, advanced_features_names = self.preprocessFeatures()
-        macro_features, macro_features_names = self.preprocessMacroFeatures()
+
         df_ret, strats, strat_features_names = self.preprocessReturns()
+        features_names = [col for col in list(advanced_features.columns) if col!='Date']
 
-        duplicated_features = [my_feat for my_feat in advanced_features_names if my_feat in macro_features]
-        advanced_features = advanced_features.drop(columns=duplicated_features)
-        advanced_features_names = [col for col in list(advanced_features.columns) if col!='Date']
+        max_date = max(advanced_features['Date'])
+        print('Actual max feature date '+str(max_date))
+        print('Actual computing date '+str(self.running_date))
 
-        features_names = advanced_features_names + macro_features_names
+        df_ret=df_ret[df_ret.index>=self.starting_date]
+        df_ret=df_ret[df_ret.index<=self.running_date]
+
+        print('Max price date '+str(max(df_ret.index)))
 
 
         # quotes_df=quotes_df.sort_values(by='date', ascending=True)
@@ -361,13 +429,12 @@ class FeaturesAssembler(AbstractAssembler):
 
         print('merging')
         df_ret = pd.merge(df_ret, advanced_features, how='left', on=['Date'])
-        df_ret = pd.merge(df_ret, macro_features, how='left', on=['Date'], right_index=True)
-
         df_ret = df_ret.replace([np.inf, -np.inf], np.nan)
         df_ret=df_ret.fillna(0.)
 
-        print('number of infs in macro features')
-        print(np.isinf(df_ret[macro_features_names]).sum(axis=0))
+        print('Final max predictor date ' + str(max(df_ret['Date'])))
+        print('Predictors size')
+        print(df_ret.shape)
 
         print('number of infs in advanced features')
         print(np.isinf(df_ret[features_names]).sum(axis=0))
@@ -378,9 +445,6 @@ class FeaturesAssembler(AbstractAssembler):
         print('number of nans in advanced features')
         print(np.isnan(df_ret[features_names]).sum(axis=0))
 
-        print('number of nans in macro features')
-        print(np.isnan(df_ret[macro_features_names]).sum(axis=0))
-
         print('number of nans in returns')
         print(np.isnan(df_ret[strats]).sum(axis=0))
 
@@ -447,7 +511,7 @@ class FeaturesAssembler(AbstractAssembler):
             raise Exception('nan values for assembled features')
         if np.isinf(features).sum(axis=0).sum() > 0:
             raise Exception('inf values for assembled features')
-
-        self.dbx.local_features_npy_save_and_upload(features = features,features_names= predictor_names,feature_type=feature_type, n_past_features = n_past_features , local_root_directory= self.local_root_directory, user = self.user, features_saving_suffix=self.features_saving_suffix, features_names_saving_suffix=self.features_names_saving_suffix)
+        print('final size saved '+str(features.shape))
+        self.dbx.local_features_npy_save_and_upload(starting_date = self.starting_date,ending_date = self.running_date, features = features,features_names= predictor_names,feature_type=feature_type, n_past_features = n_past_features , local_root_directory= self.local_root_directory, user = self.user, features_saving_suffix=self.features_saving_suffix, features_names_saving_suffix=self.features_names_saving_suffix)
 
 
