@@ -11,6 +11,7 @@ import time
 import pandas as pd
 import numpy as np
 import dropbox
+from pathlib import Path
 
 @contextlib.contextmanager
 def stopwatch(message):
@@ -34,29 +35,32 @@ class NaPoleonDropboxConnector(object):
 
     def local_sqlite_overwrite_from_db(self,folder='', subfolder='', sqlite_file_name='',local_root_directory=''):
         res = self.download(folder=folder, subfolder=subfolder, name=sqlite_file_name)
-        local_sqlite_path = local_root_directory + sqlite_file_name
-        print('overwriting local file ' + local_sqlite_path + ' with dropbox file ', sqlite_file_name)
+        local_root_directory=Path(local_root_directory)
+        local_sqlite_path = local_root_directory/subfolder/sqlite_file_name
+        print('overwriting local file ' + str(local_sqlite_path) + ' with dropbox file ', sqlite_file_name)
         f = open(local_sqlite_path, 'wb')
         f.write(res)
         f.close()
 
     def local_overwrite_and_load_pickle(self, folder='', subfolder='', returns_pkl_file_name='', local_root_directory = ''):
-        local_returns_pkl_path = local_root_directory + returns_pkl_file_name
-        print('overwriting local file ' + local_returns_pkl_path + ' with dropbox file')
+        local_root_directory = Path(local_root_directory)
+        local_returns_pkl_path = local_root_directory/subfolder/returns_pkl_file_name
+        print('overwriting local file ' + str(local_returns_pkl_path) + ' with dropbox file')
         if self.dropbox_backup:
             res = self.download(folder = folder,subfolder= subfolder, name = returns_pkl_file_name)
             f = open(local_returns_pkl_path, 'wb')
             f.write(res)
             f.close()
-        print('loading overwritting local returns pickle file reading ' + local_returns_pkl_path)
+        print('loading overwritting local returns pickle file reading ' + str(local_returns_pkl_path))
         df = pd.read_pickle(local_returns_pkl_path)
         return df
 
-    def local_features_npy_ovverwrite_and_load_array(self, feature_type=None,
+    def local_features_npy_ovverwrite_and_load_array(self, starting_date = None, ending_date = None, feature_type=None,
                                            n_past_features=None, local_root_directory='', user='',
                                            features_saving_suffix='', features_names_saving_suffix=''):
-        features_file_name = user + '_' + feature_type.name + '_' + str(n_past_features) + features_saving_suffix
-        features_full_path = local_root_directory + features_file_name
+        features_file_name = starting_date.strftime('%d_%b_%Y')+'_'+ending_date.strftime('%d_%b_%Y') +'_'+user + '_' + feature_type.name + '_' + str(n_past_features) + features_saving_suffix
+        local_root_directory = Path(local_root_directory)
+        features_full_path = local_root_directory/features_file_name
 
         if self.dropbox_backup:
             res_feat = self.download(folder='', subfolder='', name=features_file_name)
@@ -66,7 +70,7 @@ class NaPoleonDropboxConnector(object):
         features = np.load(features_full_path)
         features_names_file_names = user + '_' + feature_type.name + '_' + str(
             n_past_features) + features_names_saving_suffix
-        features_names_full_path = local_root_directory + features_names_file_names
+        features_names_full_path = local_root_directory/features_names_file_names
         if self.dropbox_backup:
             res_feat_names = self.download(folder='', subfolder='', name=features_names_file_names)
             f = open(features_names_full_path, 'wb')
@@ -75,9 +79,10 @@ class NaPoleonDropboxConnector(object):
         features_names = np.load(features_names_full_path)
         return features, features_names
 
-    def local_supervision_npy_overwrite_and_load_array(self, rebal=None, local_root_directory='', user='',supervision_npy_file_suffix=''):
-        supervision_file_name = user + '_' + str(rebal) + supervision_npy_file_suffix
-        local_supervision_npy_path = local_root_directory + supervision_file_name
+    def local_supervision_npy_overwrite_and_load_array(self, starting_date = None, ending_date = None,  rebal=None,  lower_bound=0.02, upper_bound=0.4, local_root_directory='', user='',supervision_npy_file_suffix=''):
+        supervision_file_name = starting_date.strftime('%d_%b_%Y')+'_'+ending_date.strftime('%d_%b_%Y')+'_'+user + '_' + str(rebal)  + '_' + str(lower_bound)+ '_' + str(upper_bound) +  supervision_npy_file_suffix
+        local_root_directory = Path(local_root_directory)
+        local_supervision_npy_path = local_root_directory/supervision_file_name
         if self.dropbox_backup:
             res_sup = self.download(folder='', subfolder='', name=supervision_file_name)
             f = open(local_supervision_npy_path, 'wb')
@@ -86,23 +91,25 @@ class NaPoleonDropboxConnector(object):
         results = np.load(local_supervision_npy_path)
         return results
 
-    def local_features_npy_save_and_upload(self, features=None, features_names=None, feature_type=None, n_past_features = None, local_root_directory='', user='', features_saving_suffix='', features_names_saving_suffix=''):
-        features_file_name = user + '_' + feature_type.name + '_' + str(n_past_features) + features_saving_suffix
-        features_full_path = local_root_directory  + features_file_name
-        print('saving local file '+features_full_path)
+    def local_features_npy_save_and_upload(self, starting_date = None, ending_date = None, features=None, features_names=None, feature_type=None, n_past_features = None, local_root_directory='', user='', features_saving_suffix='', features_names_saving_suffix=''):
+        features_file_name = starting_date.strftime('%d_%b_%Y')+'_'+ending_date.strftime('%d_%b_%Y')+'_'+user +'_' + feature_type.name + '_' + str(n_past_features) + features_saving_suffix
+        local_root_directory = Path(local_root_directory)
+        features_full_path = local_root_directory/features_file_name
+        print('saving local file '+str(features_full_path))
         np.save(features_full_path, features)
         if self.dropbox_backup:
             self.upload(fullname=features_full_path, folder='', subfolder='', name = features_file_name, overwrite=self.overwrite)
         features_names_file_names = user + '_' + feature_type.name + '_' + str(n_past_features) + features_names_saving_suffix
-        features_names_full_path = local_root_directory  + features_names_file_names
+        features_names_full_path = local_root_directory/features_names_file_names
         np.save(features_names_full_path, features_names)
         if self.dropbox_backup:
             self.upload(fullname=features_names_full_path, folder='', subfolder='', name = features_names_file_names, overwrite=self.overwrite)
 
-    def local_supervision_npy_save_and_upload(self, data=None, rebal = None, local_root_directory='', user='', supervision_npy_file_suffix=''):
-        supervision_file_name = user + '_' + str(rebal) + supervision_npy_file_suffix
-        local_supervision_npy_path = local_root_directory + supervision_file_name
-        print('saving local file '+local_supervision_npy_path)
+    def local_supervision_npy_save_and_upload(self, starting_date = None, data=None, rebal = None, lower_bound=0.02, upper_bound=0.4, local_root_directory='', user='', supervision_npy_file_suffix=''):
+        supervision_file_name = starting_date.strftime('%d_%b_%Y') + '_' + user + '_' + str(rebal) + '_' + str(lower_bound)+ '_' + str(upper_bound) + supervision_npy_file_suffix
+        local_root_directory = Path(local_root_directory)
+        local_supervision_npy_path = local_root_directory/supervision_file_name
+        print('saving local file '+str(local_supervision_npy_path))
         np.save(local_supervision_npy_path, data)
         if self.dropbox_backup:
             self.upload(fullname=local_supervision_npy_path, folder='', subfolder='', name = supervision_file_name, overwrite=self.overwrite)
