@@ -91,7 +91,11 @@ class _LSTMBaseNeuralNet(torch.nn.Module):
         self.optimizer.zero_grad()
         outputs = self(X)
         loss = self.criterion(outputs, y)
-
+        # if self.num_layers>1:
+        #     duplicate_y = torch.cat((y, y), 0)
+        # else:
+        #     duplicate_y = y
+        # loss = self.criterion(outputs, duplicate_y)
         loss.backward()
         self.optimizer.step()
         self.loss = loss.item()
@@ -120,14 +124,19 @@ class _LSTMBaseNeuralNet(torch.nn.Module):
         x_type, y_type : torch.dtype
             Respectively input and ouput data types. Default is `None`.
         """
+        assert x_type == y_type
+
+        if x_type == torch.float64:
+            self=self.double()
+
         if hasattr(self, 'N') and self.N != X.size(1):
             raise ValueError('X must have {} input columns'.foramt(self.N))
 
         if hasattr(self, 'M') and self.M != y.size(1):
             raise ValueError('y must have {} output columns'.format(self.M))
 
-        self.X = self._set_data(X, dtype=x_type)
-        self.y = self._set_data(y, dtype=y_type)
+        self.X = self._set_data(X)
+        self.y = self._set_data(y)
 
         if self.X.ndim == 2:
             self.T, self.N = self.X.size()
@@ -142,7 +151,7 @@ class _LSTMBaseNeuralNet(torch.nn.Module):
 
         return self
 
-    def _set_data(self, X, dtype=None):
+    def _set_data(self, X):
         """ Convert array-like data to tensor. """
         # TODO : Verify dtype of data torch tensor
         if isinstance(X, np.ndarray):
@@ -223,26 +232,16 @@ class MultiLayerLSTM(_LSTMBaseNeuralNet):
 
 
     def forward(self, x):
+        # # reset the state of LSTM
+        # # the state is kept till the end of the past sequence for the batch
+        # h_batch = Variable(torch.zeros(
+        #     self.num_layers, x.size(0), self.hidden_size))
+        # c_batch = Variable(torch.zeros(
+        #     self.num_layers, x.size(0), self.hidden_size))
+        # # Propagate input through LSTM
+        # ula, (h_out, _) = self.lstm(x, (h_batch, c_batch))
 
-        h_0 = Variable(torch.zeros(
-            self.num_layers, x.size(0), self.hidden_size))
-
-        c_0 = Variable(torch.zeros(
-            self.num_layers, x.size(0), self.hidden_size))
-
-        #
-        # h_0 = Variable(torch.zeros(
-        #     self.num_layers, self.b, self.hidden_size))
-        #
-        # c_0 = Variable(torch.zeros(
-        #     self.num_layers, self.b, self.hidden_size))
-
-        # Propagate input through LSTM
-        # print(h_0.shape)
-        # print(c_0.shape)
-        # print(x.shape)
-        #print(self)
-        ula, (h_out, _) = self.lstm(x, (h_0, c_0))
+        ula, (h_out, _) = self.lstm(x)
         #clip = 0.001
         #torch.nn.utils.clip_grad_norm(self.lstm.parameters(), clip)
 
@@ -251,6 +250,8 @@ class MultiLayerLSTM(_LSTMBaseNeuralNet):
         out = self.fc(h_out)
         #print(out.shape)
         out = self.activation(out)
+        if self.num_layers>1:
+            out = out[-x.shape[0]:]
         return out
 
 def _type_convert(dtype):
@@ -294,6 +295,9 @@ class RollMultiLayerLSTM(MultiLayerLSTM, roller._RollingBasis):
         roller._RollingBasis.__init__(self, X, y)
         MultiLayerLSTM.__init__(self, X, y, num_classes=num_classes, input_size=input_size,
                                 hidden_size=hidden_size, num_layers=num_layers,x_type = x_type, y_type = y_type, activation = activation)
+        assert x_type == y_type
+        if x_type == torch.float64:
+            self.double()
 
     def set_roll_period(self, train_period, test_period, start=0, end=None, repass_steps=1):
         """ Callable method to set target features data, and model.
