@@ -6,7 +6,6 @@ import pandas as pd
 import torch
 import torch.nn
 import numpy as np
-import torch.nn.functional as F
 import shap
 import pickle
 
@@ -15,6 +14,10 @@ from sklearn.metrics import mean_squared_error
 from napoleontoolbox.roller import roller
 from napoleontoolbox.callback import ts_callback
 from functools import partial
+from functools import reduce
+
+def list_average(lst):
+    return reduce(lambda a, b: a + b, lst) / len(lst)
 
 class _PerceptronBaseNeuralNet(torch.nn.Module):
     """ Base object for neural network model with PyTorch.
@@ -119,14 +122,20 @@ class _PerceptronBaseNeuralNet(torch.nn.Module):
         x_type, y_type : torch.dtype
             Respectively input and ouput data types. Default is `None`.
         """
+
+        assert x_type == y_type
+
+        if x_type == torch.float64:
+            self = self.double()
+
         if hasattr(self, 'N') and self.N != X.size(1):
             raise ValueError('X must have {} input columns'.foramt(self.N))
 
         if hasattr(self, 'M') and self.M != y.size(1):
             raise ValueError('y must have {} output columns'.format(self.M))
 
-        self.X = self._set_data(X, dtype=x_type)
-        self.y = self._set_data(y, dtype=y_type)
+        self.X = self._set_data(X)
+        self.y = self._set_data(y)
         self.T, self.N = self.X.size()
         T_veri, self.M = self.y.size()
 
@@ -136,7 +145,7 @@ class _PerceptronBaseNeuralNet(torch.nn.Module):
 
         return self
 
-    def _set_data(self, X, dtype=None):
+    def _set_data(self, X):
         """ Convert array-like data to tensor. """
         # TODO : Verify dtype of data torch tensor
         if isinstance(X, np.ndarray):
@@ -233,14 +242,12 @@ class MultiLayerPerceptron(_PerceptronBaseNeuralNet):
     def forward(self, x):
         """ Forward computation. """
         x = self.drop(x)
-        for layer in self.layers:
-            x = self.activation(layer(x))
-        # for name, layer in enumerate(self.layers):
-        #     if name == (self.n_layers) and self.activation.__name__ == F.relu.__name__:
-        #         x = layer(x)
-        #     else:
-        #         x = self.activation(layer(x))
-
+        self.handle_callback('after_layer', layer_index=0, layer_activ_mean=x.data.mean().item(),
+                             layer_activ_std=x.data.std().item())
+        for layer_index, layer in enumerate(self.layers):
+            x = layer(x)
+            x = self.activation(x)
+            self.handle_callback('after_layer',layer_index = layer_index+1, layer_activ_mean=x.data.mean().item(), layer_activ_std = x.data.std().item())
         return x
 
 
@@ -291,6 +298,8 @@ class RollMultiLayerPerceptron(MultiLayerPerceptron, roller._RollingBasis):
                                       activation=activation, drop=drop,
                                       x_type=x_type, y_type=y_type,
                                       activation_kwargs=activation_kwargs)
+        if x_type == torch.float64:
+            self.double()
 
 
     def set_roll_period(self, train_period, test_period, start=0, end=None,repass_steps=1):
@@ -330,15 +339,46 @@ class RollMultiLayerPerceptron(MultiLayerPerceptron, roller._RollingBasis):
         """ Predict. """
         return self.predict(X=X)
 
-    def eval_predictor_importance(self, features, features_names):
+    def eval_predictor_importance(self, features, actual_predictions, features_names, check = True):
+        deep_explainer = shap.DeepExplainer(self, features)
+        shap_values = deep_explainer.shap_values(features)
+        expected_values = deep_explainer.expected_value
+        if check :
+            for weight_index in range(actual_predictions.shape[1]):
+                for prediction_index in range(actual_predictions.shape[0]):
+                    weight_shap_values = shap_values[weight_index]
+                    weight_expected_values = expected_values[weight_index]
+                    weight_actual_predictions = actual_predictions.numpy()[:,weight_index]
+                    weight_feature_effect = weight_shap_values.sum(axis=1)
+                    weight_actual_predictions_from_shapley_values = weight_expected_values + weight_feature_effect
+                    if np.sum(np.abs(weight_actual_predictions - weight_actual_predictions_from_shapley_values)) > 1e-6:
+                        raise Exception('Mismatch between shapley values and actual predictions')
+
+        predictors_shap_values = [list_average(shap_values)]
+        predictors_feature_order = np.argsort(np.sum(np.mean(np.abs(predictors_shap_values), axis=0), axis=0))
+
+        predictors_left_pos = np.zeros(len(predictors_feature_order))
+
+        predictors_class_inds = np.argsort([-np.abs(predictors_shap_values[i]).mean() for i in range(len(predictors_shap_values))])
+        for i, ind in enumerate(predictors_class_inds):
+            predictors_global_shap_values = np.abs(predictors_shap_values[ind]).mean(0)
+            predictors_left_pos += predictors_global_shap_values[predictors_feature_order]
+
+        predictors_ds = {}
+        predictors_ds['features'] = np.asarray(features_names)[predictors_feature_order]
+        predictors_ds['values'] = predictors_left_pos
+        predictors_features_df = pd.DataFrame.from_dict(predictors_ds)
+        values = {}
+        for index, row in predictors_features_df.iterrows():
+            values[row['features']]=row['values']
+
+        return values
+
+    def gradient_eval_predictor_importance(self, features, actual_predictions, features_names, display = False):
         explainer_shap = shap.GradientExplainer(model=self,
                                             data=features)
         # Fit the explainer on a subset of the data (you can try all but then gets slower)
-
-        shap_values = explainer_shap.shap_values(X=features,nsamples = 100, ranked_outputs=True,output_rank_order='max',rseed=None, return_variances=False)
-        #shap_values = explainer_shap.shap_values(X=features,
-        #                                         ranked_outputs=True)
-
+        shap_values = explainer_shap.shap_values(features,nsamples = 500, ranked_outputs=True,output_rank_order='max',rseed=None,return_variances=False)
         predictors_shap_values = shap_values[0]
         predictors_feature_order = np.argsort(np.sum(np.mean(np.abs(predictors_shap_values), axis=0), axis=0))
 
@@ -401,29 +441,36 @@ class RollMultiLayerPerceptron(MultiLayerPerceptron, roller._RollingBasis):
         self.last_test_slice = test_slice
         return eval_slice, test_slice
 
-    def unroll_features(self,dates, feature_names):
+    def unroll_features(self, dates, feature_names, display_slices = False, gradient = False):
+        self.handle_callback('begin_fit')
         rows_list = []
         dates_list = []
+        prediction_list = []
+        truth_list = []
         for eval_slice, test_slice in self:
-            print()
             # Compute prediction on eval and test set
-            self.y_eval[eval_slice] = self.sub_predict(self.X[eval_slice])
+            eval_prediction = self.sub_predict(self.X[eval_slice])
+            self.y_eval[eval_slice] = eval_prediction
             test_prediction = self.sub_predict(self.X[test_slice])
             self.y_test[test_slice] = test_prediction
-            # getting the features importance
-            features_dico = self.eval_predictor_importance(self.X[test_slice], feature_names)
-
+            ### features computation
+            if gradient :
+                features_dico = self.gradient_eval_predictor_importance(self.X[test_slice], test_prediction, feature_names)
+            else:
+                features_dico = self.eval_predictor_importance(self.X[test_slice], test_prediction, feature_names)
             rows_list.append(features_dico)
             dates_list.append(dates[test_slice.start])
 
-            # Update loss function of eval set and test set
-
+            ### loss computation
             ev = self.y_eval[eval_slice]
             ev_true = self.y[eval_slice]
 
             tt = self.y_test[test_slice]
             tt_true = self.y[test_slice]
 
+            prediction_list.append(self.y_test[test_slice.start])
+            truth_list.append(self.y[test_slice.start])
+
             self.loss_eval += [mean_squared_error(ev, ev_true)]
             self.loss_test += [mean_squared_error(tt, tt_true)]
 
@@ -432,13 +479,28 @@ class RollMultiLayerPerceptron(MultiLayerPerceptron, roller._RollingBasis):
             txt = '{:5.2%} is done | '.format(pct)
             txt += 'Eval loss is {:5.2} | '.format(self.loss_eval[-1])
             txt += 'Test loss is {:5.2} | '.format(self.loss_test[-1])
-            if np.random.rand()>=0.9:
+            if np.random.rand()>=0.7:
                 print(txt)
-
+                if display_slices:
+                    print('eval slice ' + str(eval_slice))
+                    print('test slice ' + str(test_slice))
+                    print('adding date '+ str(dates[test_slice.start]))
+                    if self.X[eval_slice].shape == self.X[test_slice].shape:
+                        m = (self.X[eval_slice] - self.X[test_slice])
+                        distinct_frobenius = (m * m).sum().sqrt()
+                        print('distinct frobenius between predictors ' + str(distinct_frobenius))
+                        t = test_prediction - eval_prediction
+                        distinct_frobenius_pred = (t * t).sum().sqrt()
+                        print('distinct frobenius between predictions ' + str(distinct_frobenius_pred))
+                    if abs(test_prediction.numpy().sum()) <= 1e-6:
+                        print('null prediction, investigate')
         features_df = pd.DataFrame(rows_list)
         features_df.index = list(dates_list)
-
-        return features_df
+        predictions_df = pd.DataFrame(prediction_list)
+        returns_df = pd.DataFrame(truth_list)
+        predictions_df.index = list(dates_list)
+        returns_df.index = list(dates_list)
+        return features_df, predictions_df, returns_df
 
     def print_parameters(self):
         # Print model's state_dict
