@@ -8,8 +8,11 @@ from multiprocessing import Pool
 from napoleontoolbox.features import features_type
 from napoleontoolbox.file_saver import dropbox_file_saver
 from napoleontoolbox.neural_net import lrs_type
+from napoleontoolbox.neural_net import activations
+
+
 class ParalleLauncher():
-    def __init__(self,drop_token='', local_root_directory='../data',user='napoleon',  db_path_suffix = '_run.sqlite', meta_features_types=[features_type.FeaturesType.STANDARD_ADVANCED], meta_layers=[[2048,1024,512,256,128,64,32]], convolutions=[0], activations=['relu'], meta_n=[126], meta_seeds=[0], epochss=[1], ss=[21], meta_n_past_features=[21], utilities=['f_drawdown'], meta_lrs_types = [lrs_type.LrType.CONSTANT],meta_lrs = [0.01]):
+    def __init__(self,drop_token='', dropbox_backup=True, local_root_directory='../data',user='napoleon',  db_path_suffix = '_run.sqlite', meta_features_types=[features_type.FeaturesType.STANDARD_ADVANCED], meta_layers=[[2048,1024,512,256,128,64,32]], convolutions=[0], activations=[activations.ActivationsType.LEAK_0SUB_0MAX0], meta_n=[126], meta_seeds=[0], epochss=[1], ss=[21], meta_n_past_features=[21], utilities=['f_drawdown'], meta_lrs_types = [lrs_type.LrType.CONSTANT],meta_lrs = [0.01],lower_bounds=[0.02],upper_bounds=[0.4], meta_normalize=[True]):
         # print('Epochs ' + str(epochss))
         # print('Seed ' + str(meta_seeds))
         # print('Training period size ' + str(meta_n))
@@ -28,22 +31,27 @@ class ParalleLauncher():
         # print('Rebalancing ' + str(ss))
         self.args = []
         self.counter = 1
-        for lrs_type in meta_lrs_types:
-            for lr in meta_lrs:
-                for utility in utilities:
-                    for n_past_features in meta_n_past_features:
-                        for convolution in convolutions:
-                            for activation in activations:
-                                for s in ss:
-                                    for epochs in epochss:
-                                        for feature_type in meta_features_types:
-                                            for seed in meta_seeds:
-                                                for layers in meta_layers:
-                                                    for n in meta_n:
-                                                        self.args.append((self,seed, utility, layers, epochs,
-                                                                     n_past_features, n, s, feature_type,
-                                                                     activation, convolution,lr, lrs_type))
-                                                        self.counter = self.counter + 1
+
+        for n_past_features in meta_n_past_features:
+            for convolution in convolutions:
+                for activation in activations:
+                    for s in ss:
+                        for epochs in epochss:
+                            for feature_type in meta_features_types:
+                                for seed in meta_seeds:
+                                    for layers in meta_layers:
+                                        for n in meta_n:
+                                            for low_bound in lower_bounds:
+                                                for up_bound in upper_bounds:
+                                                    for normalize in meta_normalize:
+                                                        for lrs_type in meta_lrs_types:
+                                                            for lr in meta_lrs:
+                                                                for utility in utilities:
+                                                                    self.args.append((self,seed, utility, layers, epochs,
+                                                                                 n_past_features, n, s, feature_type,
+                                                                                 activation, convolution,lr, lrs_type, low_bound, up_bound, normalize))
+                                                                    self.counter = self.counter + 1
+        self.args.sort()
         self.local_root_directory = local_root_directory
         self.user = user
         self.db_path_suffix = db_path_suffix
@@ -53,7 +61,7 @@ class ParalleLauncher():
         self.totalRow = 0
         self.instantiateTable()
         self.empty_runs_to_investigate = []
-        self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token)
+        self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=drop_token,dropbox_backup=dropbox_backup)
 
     def launchParallelPool(self, toRun, use_num_cpu):
         with Pool(processes=use_num_cpu) as pool:
@@ -61,8 +69,11 @@ class ParalleLauncher():
             print('parallel computation done')
 
     def launchSequential(self, toRun):
+        results=[]
         for meArg in self.args:
-            toRun(*meArg)
+            predicted_weights = toRun(*meArg)
+            results.append(predicted_weights)
+        return results
 
     def instantiateTable(self):
         sqliteConnection = None
@@ -171,18 +182,54 @@ class ParalleLauncher():
                 print("sqlite connection is closed")
 
 
+    def addRunActivationsFollowUp(self, run, activations_df):
+        sqliteConnection = None
+        try:
+            table_name = run + '_activation'
+            sqliteConnection = sqlite3.connect(self.db_path)
+            activations_df.to_sql(name=table_name, con=sqliteConnection)
+        except sqlite3.Error as error:
+            print("Error while creating a sqlite table", error)
+        finally:
+            if (sqliteConnection):
+                sqliteConnection.close()
+                print("sqlite connection is closed")
+
     def addResults(self, run, values):
         success = self.insertResults(run,values)
         if not success :
             self.updateResults(run, values)
 
 
-    def saveResults(self, run, values, weights_df, recordings_df):
+    def saveResults(self, run, values, weights_df, recordings_df, activations_df):
         self.addRun(run)
         self.addResults(run, values)
         self.totalRow = self.datesCount()
         self.addRunWeights(run, weights_df)
         self.addRunRecordings(run, recordings_df)
+        self.addRunActivationsFollowUp(run, activations_df)
+
+    def checkRunExistence(self,run):
+        sqliteConnection = None
+        runExistence=None
+        try:
+            sqliteConnection = sqlite3.connect(self.db_path)
+            sqlite_select_run_query = 'SELECT effective_date,'+run + ' FROM parallel_run'
+            cursor = sqliteConnection.cursor()
+            print("Successfully Connected to SQLite")
+            cursor.execute(sqlite_select_run_query)
+            sqliteConnection.commit()
+            print("SQLite table created")
+            cursor.close()
+            runExistence = True
+        except sqlite3.Error as error:
+            print("The run is not present", error)
+            runExistence = False
+        finally:
+            if (sqliteConnection):
+                sqliteConnection.close()
+                print("sqlite connection is closed")  # adding the column
+        return runExistence
 
     def datesCount(self):
         sqliteConnection = None
