@@ -181,8 +181,7 @@ def calmar(series, period=252):
     annual_return = np.sign(ret) * np.float_power(
         np.abs(ret), period / len(series), dtype=np.float64) - 1.
     # Compute MaxDrawDown
-    max_dd = mdd(series)
-
+    max_dd = max(mdd(series),10e-6)
     return annual_return / max_dd
 
 
